#!/usr/bin/env node
/**
 * Copyright 2019 Google LLC. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const coerce_option_1 = require("../util/coerce-option");
const github_release_1 = require("../github-release");
const release_pr_1 = require("../release-pr");
const java_auth_yoshi_1 = require("../releasers/java-auth-yoshi");
const node_1 = require("../releasers/node");
const php_yoshi_1 = require("../releasers/php-yoshi");
const ruby_yoshi_1 = require("../releasers/ruby-yoshi");
const java_yoshi_1 = require("../releasers/java-yoshi");
const yargs = require('yargs');
const argv = yargs
    .command('release-pr', 'create or update a PR representing the next release', (yargs) => {
    yargs
        .option('package-name', {
        describe: 'name of package release is being minted for',
        demand: true,
    })
        .option('last-package-version', {
        describe: 'last version # that package was released as',
    })
        .option('repo-url', {
        describe: 'GitHub URL to generate release for',
        demand: true,
    })
        .option('label', {
        describe: 'label(s) to add to generated PR',
    })
        .option('snapshot', {
        describe: 'is it a snapshot (or pre-release) being generated?',
        type: 'boolean',
        default: false,
    });
}, (argv) => {
    let rp;
    switch (argv.releaseType) {
        case release_pr_1.ReleaseType.Node:
            rp = new node_1.Node(argv);
            break;
        case release_pr_1.ReleaseType.PHPYoshi:
            rp = new php_yoshi_1.PHPYoshi(argv);
            break;
        case release_pr_1.ReleaseType.JavaAuthYoshi:
            // TODO: coerce this to the generic Java release
            rp = new java_auth_yoshi_1.JavaAuthYoshi(argv);
            break;
        case release_pr_1.ReleaseType.JavaYoshi:
            rp = new java_yoshi_1.JavaYoshi(argv);
            break;
        case release_pr_1.ReleaseType.RubyYoshi:
            rp = new ruby_yoshi_1.RubyYoshi(argv);
            break;
        default:
            throw Error('unknown release type');
    }
    rp.run().catch(handleError);
})
    .command('github-release', 'create a GitHub release from a release PR', (yargs) => {
    yargs
        .option('package-name', {
        describe: 'name of package release is being minted for',
        demand: true,
    })
        .option('repo-url', {
        describe: 'GitHub URL to generate release for',
        demand: true,
    })
        .option('label', {
        default: 'autorelease: pending',
        describe: 'label to remove from release PR',
    });
}, (argv) => {
    const gr = new github_release_1.GitHubRelease(argv);
    gr.createRelease().catch(handleError);
})
    .command('generate-action', 'outputs the release-please stanzas that should be added to main.workflow', (yargs) => {
    yargs.option('package-name', {
        describe: 'name of package releases will be created for',
        demand: true,
    });
}, (argv) => {
    console.error(chalk_1.default.green('----- put the content below in .github/main.workflow -----'));
    console.info(`workflow "Groom Release PR" {
  on = "pull_request"
  resolves = ["release-pr"]
}

action "release-pr" {
  uses = "googleapis/release-please/.github/action/release-please@master"
  env = {
    PACKAGE_NAME = "${argv.packageName}"
    RELEASE_PLEASE_COMMAND = "release-pr"
  }
  secrets = ["GITHUB_TOKEN"]
}

workflow "GitHub Release" {
  on = "pull_request"
  resolves = ["github-release"]
}

action "github-release" {
  uses = "googleapis/release-please/.github/action/release-please@master"
  env = {
    PACKAGE_NAME = "${argv.packageName}"
    RELEASE_PLEASE_COMMAND = "github-release"
  }
  secrets = ["GITHUB_TOKEN"]
}
        `);
})
    .middleware((argv) => {
    // allow secrets to be loaded from file path
    // rather than being passed directly to the bin.
    if (argv.token)
        argv.token = coerce_option_1.coerceOption(argv.token);
    if (argv.apiUrl)
        argv.apiUrl = coerce_option_1.coerceOption(argv.apiUrl);
    if (argv.proxyKey)
        argv.proxyKey = coerce_option_1.coerceOption(argv.proxyKey);
})
    .option('token', { describe: 'GitHub token with repo write permissions' })
    .option('release-as', {
    describe: 'override the semantically determined release version',
    type: 'string',
})
    .option('release-type', {
    describe: 'what type of repo is a release being created for?',
    choices: ['node', 'php-yoshi', 'java-auth-yoshi', 'ruby-yoshi'],
    default: 'node',
})
    .option('bump-minor-pre-major', {
    describe: 'should we bump the semver minor prior to the first major release',
    default: false,
    type: 'boolean',
})
    .option('api-url', {
    describe: 'URL to use when making API requests',
    default: 'https://api.github.com',
    type: 'string',
})
    .option('proxy-key', {
    describe: 'key used by some GitHub proxies',
    type: 'string',
})
    .option('debug', {
    describe: 'print verbose errors (use only for local debugging).',
    default: false,
    type: 'boolean',
})
    .demandCommand(1)
    .strict(true)
    .parse();
// The errors returned by octokit currently contain the
// request object, this contains information we don't want to
// leak. For this reason, we capture exceptions and print
// a less verbose error message (run with --debug to output
// the request object, don't do this in CI/CD).
function handleError(err) {
    let status = '';
    const command = argv._.length === 0 ? '' : argv._[0];
    if (err.status) {
        status = '' + err.status;
    }
    console.error(chalk_1.default.red(`command ${command} failed${status ? ` with status ${status}` : ''}`));
    if (argv.debug) {
        console.error('---------');
        console.error(err.stack);
    }
    process.exitCode = 1;
}
process.on('unhandledRejection', err => {
    handleError(err);
});
process.on('uncaughtException', err => {
    handleError(err);
});
//# sourceMappingURL=release-please.js.map