"use strict";
/**
 * Copyright 2019 Google LLC. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const semver = require("semver");
const checkpoint_1 = require("./util/checkpoint");
const github_1 = require("./github");
const parseGithubRepoUrl = require('parse-github-repo-url');
var ReleaseType;
(function (ReleaseType) {
    ReleaseType["Node"] = "node";
    ReleaseType["PHPYoshi"] = "php-yoshi";
    ReleaseType["JavaAuthYoshi"] = "java-auth-yoshi";
    ReleaseType["JavaYoshi"] = "java-yoshi";
    ReleaseType["RubyYoshi"] = "ruby-yoshi";
})(ReleaseType = exports.ReleaseType || (exports.ReleaseType = {}));
const DEFAULT_LABELS = 'autorelease: pending,type: process';
class ReleasePR {
    constructor(options) {
        this.bumpMinorPreMajor = options.bumpMinorPreMajor || false;
        this.labels = options.label
            ? options.label.split(',')
            : DEFAULT_LABELS.split(',');
        this.repoUrl = options.repoUrl;
        this.token = options.token;
        this.packageName = options.packageName;
        this.releaseAs = options.releaseAs;
        this.releaseType = options.releaseType;
        this.apiUrl = options.apiUrl;
        this.proxyKey = options.proxyKey;
        this.snapshot = options.snapshot;
        // drop a `v` prefix if provided:
        this.lastPackageVersion = options.lastPackageVersion
            ? options.lastPackageVersion.replace(/^v/, '')
            : undefined;
        this.gh = this.gitHubInstance(options.octokitAPIs);
    }
    async run() {
        const pr = await this.gh.findMergedReleasePR(this.labels);
        if (pr) {
            // a PR already exists in the autorelease: pending state.
            checkpoint_1.checkpoint(`pull #${pr.number} ${pr.sha} has not yet been released`, checkpoint_1.CheckpointType.Failure);
        }
        else {
            return this._run();
        }
    }
    async _run() {
        throw Error('must be implemented by subclass');
    }
    async closeStaleReleasePRs(currentPRNumber, includePackageName = false) {
        const prs = await this.gh.findOpenReleasePRs(this.labels);
        for (let i = 0, pr; i < prs.length; i++) {
            pr = prs[i];
            // don't close the most up-to-date release PR.
            if (pr.number !== currentPRNumber) {
                // on mono repos that maintain multiple open release PRs, we use the
                // pull request title to differentiate between PRs:
                if (includePackageName && !pr.title.includes(this.packageName)) {
                    continue;
                }
                checkpoint_1.checkpoint(`closing pull #${pr.number}`, checkpoint_1.CheckpointType.Failure);
                await this.gh.closePR(pr.number);
            }
        }
    }
    async coerceReleaseCandidate(cc, latestTag) {
        const previousTag = latestTag ? latestTag.name : undefined;
        let version = latestTag ? latestTag.version : '1.0.0';
        if (latestTag && !this.releaseAs) {
            const bump = await cc.suggestBump(version);
            const candidate = semver.inc(version, bump.releaseType);
            if (!candidate)
                throw Error(`failed to increment ${version}`);
            version = candidate;
        }
        else if (this.releaseAs) {
            version = this.releaseAs;
        }
        return { version, previousTag };
    }
    async commits(sha, perPage = 100, labels = false, path = null) {
        const commits = await this.gh.commitsSinceSha(sha, perPage, labels, path);
        if (commits.length) {
            checkpoint_1.checkpoint(`found ${commits.length} commits since ${sha}`, checkpoint_1.CheckpointType.Success);
        }
        else {
            checkpoint_1.checkpoint(`no commits found since ${sha}`, checkpoint_1.CheckpointType.Failure);
        }
        return commits;
    }
    gitHubInstance(octokitAPIs) {
        const [owner, repo] = parseGithubRepoUrl(this.repoUrl);
        return new github_1.GitHub({
            token: this.token,
            owner,
            repo,
            apiUrl: this.apiUrl,
            proxyKey: this.proxyKey,
            octokitAPIs,
        });
    }
    async openPR(sha, changelogEntry, updates, version, includePackageName = false) {
        const title = includePackageName
            ? `Release ${this.packageName} ${version}`
            : `chore: release ${version}`;
        const body = `:robot: I have created a release \\*beep\\* \\*boop\\* \n---\n${changelogEntry}\n\nThis PR was generated with [Release Please](https://github.com/googleapis/release-please).`;
        const pr = await this.gh.openPR({
            branch: includePackageName
                ? `release-${this.packageName}-v${version}`
                : `release-v${version}`,
            version,
            sha,
            updates,
            title,
            body,
            labels: this.labels,
        });
        // a return of -1 indicates that PR was not updated.
        if (pr > 0) {
            await this.gh.addLabels(pr, this.labels);
            await this.closeStaleReleasePRs(pr, includePackageName);
        }
    }
    changelogEmpty(changelogEntry) {
        return changelogEntry.split('\n').length === 1;
    }
}
exports.ReleasePR = ReleasePR;
//# sourceMappingURL=release-pr.js.map